import json
from pathlib import Path

import requests
from dotenv import load_dotenv
import epdx
import os

from shared import get_folder, ECOPLATFORM_URL, read_epdx_files, organize_data, visualize_histogram


def get_epds(search: dict[str, str] | None = None, limit=10) -> dict:
    """Get EPDs from ECO Platform"""

    response = requests.get(
        f"{ECOPLATFORM_URL}/processes?"
        f"format=json"
        f"&pageSize={limit}"
        f"&distributed=true&virtual=true"
        f"{'&search=true&' + '&'.join([f'{key}={value}' for key, value in search.items()]) if search else ''}",
        headers={
            'Content-Type': 'application/json',
            "Authorization": f"Bearer {os.getenv('API_TOKEN')}"
        }
    )
    response.raise_for_status()
    data = response.json()

    print(
        f"Retrieved {data.get('pageSize')} EPDs out of {data.get('totalCount')} from Ökobau"
    )

    return data


def get_full_epd(url: str) -> dict:
    """Get the full dataset for a single EPD"""

    response = requests.get(f"{url}?format=json&view=extended")

    response.raise_for_status()
    data = response.json()
    data["source"] = url

    return data


def download_to_epdx(data_folder: Path, search_key: str, limit: int = 50):
    """Download 50 EPDs and save them locally as EPDx files"""

    data = get_epds({"name": search_key}, limit=limit)
    print(f"Downloading {limit} EPDs\n")

    for _epd in data.get("data"):
        epd_id = _epd.get("uuid")
        epd_url = _epd.get("uri")

        epdx_file_path = data_folder / f"{epd_id}.epdx.json"
        if not epdx_file_path.exists():
            print(f"Downloading EPD {epd_id[:8]} to {'/'.join(data_folder.parts[-3:])}")

            # Get full EPD from ECO Platform
            epd = get_full_epd(epd_url)

            # Turn EPD into EPDx JSON string that can be saved to disk.
            try:
                epdx_data = epdx.convert_ilcd(epd, as_type=str)
                epdx_file_path.write_text(epdx_data)
            except epdx.ParsingException as err:
                print(f"Error ID: {epd_id} - {err}")
                (data_folder / f"{epd_id}.json").write_text(json.dumps(epd))
        else:
            print(f"EPD {epd_id[:8]} already in {'/'.join(data_folder.parts[-3:])}")


def visualize_ecoplatform(download_folder: Path, search_key: str):
    """Get EPDs from ECO Platform and Visualize them"""

    download_to_epdx(download_folder, search_key)
    epdx_data = read_epdx_files(download_folder)

    names, impact_category_values = organize_data(epdx_data)
    visualize_histogram(impact_category_values)


if __name__ == "__main__":
    _folder = get_folder(__file__, "data")
    load_dotenv()
    visualize_ecoplatform(_folder, "beton")
