import json
from pathlib import Path
import matplotlib.pyplot as plt
import epdx
from epdx import ImpactCategory

ECOPLATFORM_URL = "https://data.eco-platform.org/resource/datastocks/5c4cf6c3-3cdb-4688-800f-d595c4600b10"


def get_folder(source, name: str) -> Path:
    """Get or create a folder with the given name"""

    folder = Path(source).parent / name
    if not folder.exists():
        folder.mkdir()

    return folder


def read_epdx_files(download_folder: Path) -> list[epdx.EPD]:
    """Reads a folder of EPDx formatted json files in to EPDx Pydantic classes"""

    epdx_data = []
    for file in download_folder.iterdir():
        if str(file).endswith(".epdx.json"):
            epdx_data.append(epdx.EPD(**json.loads(file.read_text())))

    return epdx_data


def visualize_histogram(impact_category_values: dict[str, list]):
    """Histogram of GWP A1-A3 Values"""

    fig, ax = plt.subplots()

    gwp_a1a3 = [value for value in impact_category_values["a1a3"] if value]
    ax.hist(gwp_a1a3)

    ax.set_title("GWP values for A1-A3 per declared unit")
    ax.set_xlabel("kg-CO₂-Eq. per declared unit")
    plt.show()


def organize_data(epdx_data: list[epdx.EPD]) -> tuple[list[str], dict[str, list]]:
    """Organized data for charts"""
    names = []
    impact_category_values = {key: [] for key in ImpactCategory().__fields__.keys()}

    for epd in epdx_data:
        names.append(epd.name)
        for impact_category in impact_category_values.keys():
            try:
                gwp_value = getattr(epd.gwp, impact_category) or 0.0
            except AttributeError:
                gwp_value = 0.0

            impact_category_values[impact_category].append(gwp_value)

    return names, impact_category_values
